extends Node2D

onready var data_copy=get_node("../data_copy")
onready var data_dyn=get_node("../data_dyn")

# Notice that each Material is unique for every buffer and Layer because they use unique textures

func _ready():
  randomize()
  # binding copy of buffer back to buffer
  for a in range(data_dyn.get_child_count()):
    data_dyn.get_child(a).get_node("Sprite").material.set("shader_param/iChannel0",data_copy.get_child(a).get_texture())
    data_dyn.get_child(a).get_node("Sprite").material.set("shader_param/layer_id",a)
    var texture=init_texture(a)
    data_dyn.get_child(a).get_node("Sprite").material.set("shader_param/iChannel1",texture)
    get_child(a).material.set("shader_param/iChannel2",texture)
  

func get_index2(idx, size):
  return Vector2(int(idx)%int(size.x),int(int(idx)/int(size.x)))

# return texture with data per pixel [pos.xy, scale, tile_id]
func init_texture(layer_id):
  var num_particles=20
  var particle_size=64
  var tile_texture_map_size=Vector2(256,192)/32 # (256,192) is tiles.png size
  var iResolution=get_viewport().size
  
  var data=[] # array of vec4
  var array_width = int(sqrt(num_particles))
  var array_heigh = int(sqrt(num_particles))+1
  for a in range(num_particles):
    var IDX=get_index2(a,Vector2(sqrt(num_particles),sqrt(num_particles)))
    var pos=Vector2()
    pos.x=float(IDX.x)*particle_size*3.5
    pos.x+=iResolution.x/2-0.5*particle_size*3.5*sqrt(num_particles)
    
    pos.y=float(IDX.y)*particle_size*1.5
    pos.y+=iResolution.y/2-0.5*particle_size*1.5*sqrt(num_particles)
    pos+=float(layer_id)*Vector2(float(particle_size)*0.5,float(particle_size)*0.5)
    
    data.append(pos.x)
    data.append(pos.y)
    
    var rand = randf()
    data.append(0.20+0.80*rand) #scale
    var max_tile_texture_id=tile_texture_map_size.y*tile_texture_map_size.x;
    data.append(max_tile_texture_id*rand)
  
  var byte_array = PoolByteArray()
  var real_arr=PoolRealArray(data)
  
  for a in range(real_arr.size()):
    var spb = StreamPeerBuffer.new()
    spb.put_float(real_arr[a])
    spb.seek(0)
    byte_array.append_array(spb.get_data(4)[1])
  
  var img = Image.new()
  img.create_from_data(array_width, array_heigh, false, Image.FORMAT_RGBAF, byte_array)
  var texture = ImageTexture.new()
  texture.create_from_image(img, 0)
  return texture

var reset=true

func _process(delta):
  var iResolution=get_viewport().size
  var iMouse=Vector3()
  iMouse.x=get_viewport().get_mouse_position().x
  iMouse.y=get_viewport().get_mouse_position().y
  iMouse.z=0
  if(Input.is_mouse_button_pressed(BUTTON_LEFT)):
    iMouse.z=1
  if(Input.is_mouse_button_pressed(BUTTON_RIGHT)):
    iMouse.z=-1
  for a in range(data_dyn.get_child_count()):
    data_dyn.get_child(a).get_node("Sprite").material.set("shader_param/iResolution",iResolution)
    data_dyn.get_child(a).get_node("Sprite").material.set("shader_param/iMouse",iMouse)
    data_dyn.get_child(a).get_node("Sprite").material.set("shader_param/reset",reset)
    if(reset):
      var texture=init_texture(a)
      data_dyn.get_child(a).get_node("Sprite").material.set("shader_param/iChannel1",texture)
      get_child(a).material.set("shader_param/iChannel2",texture)
  reset=false



func _on_reset_pressed():
  reset=true
