shader_type particles;
render_mode keep_data,disable_velocity,disable_force;

// in this example I use CUSTOM transform feedback value to save and read data
// CUSTOM data RGBA [velX, velY, scale, image_tile_id]

// you can use sampler2D here for reading data and using CUSTOM as aditional storage

// you can not read/write CUSTOM data from anywhere else except this particle

// https://docs.godotengine.org/en/stable/tutorials/shading/shading_reference/particle_shader.html

uniform vec3 iMouse; // [x,y,0 or 1 if left click or -1 if it right click]
uniform vec2 iResolution;
uniform bool reset;
uniform int layer_id;  // used to control vel multiplier

const ivec2 tile_texture_map_size=ivec2(256,192)/32; // (256,192) is tiles.png size

const float num_particles=20.; // number of particles total
const float particle_size=64.; // in pixels

const float max_vel=4.; // in pixels per frame 1 pixel per frame means 60 pixels per sec
const float vel_slowdown=1.5; // in % example 5 means 5% slowdown every frame

const float pi = 3.1415926;
const float degree_to_rad = pi / 180.0;

// random
float rand_from_seed(inout uint seed) {
    int k;
    int s = int(seed);
    if (s == 0)
    s = 305420679;
    k = s / 127773;
    s = 16807 * (s - k * 127773) - 2836 * k;
    if (s < 0)
        s += 2147483647;
    seed = uint(s);
    return float(seed % uint(65536)) / 65535.0;
}

uint hash(uint x) {
    x = ((x >> uint(16)) ^ x) * uint(73244475);
    x = ((x >> uint(16)) ^ x) * uint(73244475);
    x = (x >> uint(16)) ^ x;
    return x;
}

int get_index(in ivec2 idx, in ivec2 size){
    return (idx.x%size.x)+idx.y*size.x;
}

ivec2 get_index2(in int idx, in ivec2 size){
    return ivec2(idx%size.x,idx/size.x);
}

void vertex() {
    
    
    // I use CUSTOM.w as init, CUSTOM==vec4(0.) on first frame
    if((CUSTOM.w==0.)||(reset))
    {
      //init
      CUSTOM.xy=vec2(0.); //init vel
      
      uint hs=hash(NUMBER + uint(1+INDEX) + RANDOM_SEED+uint(TIME*100.));
      float rand = rand_from_seed(hs);
      CUSTOM.z=0.20+0.80*rand; //scale
      
      const int max_tile_texture_id=tile_texture_map_size.y*tile_texture_map_size.x;
      CUSTOM.w=1.+float(max_tile_texture_id)*rand; // texture tile shifted by +1 because I use 0 to init
      
      vec2 pos=vec2(0.); //init pos
      ivec2 IDX=get_index2(INDEX,ivec2(int(sqrt(num_particles))));
      pos.x=float(IDX.x)*particle_size*3.5;
      pos.x+=iResolution.x/2.-0.5*particle_size*3.5*sqrt(num_particles);
      
      pos.y=float(IDX.y)*particle_size*1.5;
      pos.y+=iResolution.y/2.-0.5*particle_size*1.5*sqrt(num_particles);
      pos+=float(layer_id)*vec2(float(particle_size)*0.5);
      
      TRANSFORM = EMISSION_TRANSFORM;
      TRANSFORM[3].xyz=vec3(pos.x,pos.y,0.);
      
      //do not control scale in transform better to do it in vertex shader of particle
      TRANSFORM[0].xyz *= 1.;
      TRANSFORM[1].xyz *= 1.;
      TRANSFORM[2].xyz *= 1.;
      
     }
    else{
      
      vec2 pos=TRANSFORM[3].xy;
      vec2 vel=CUSTOM.xy;
      pos+=vel;
      
      
      //slowdown vel
      vel*=1.-vel_slowdown/100.;
      
      //mouse clicked
      if(iMouse.z>0.5){
        float d=1.-smoothstep(particle_size,particle_size*8.,length(iMouse.xy-pos)); //distance to mouse click 
        vec2 dir=vec2(1.,0.); 
        if(length(iMouse.xy-pos)>0.001)dir=normalize(iMouse.xy-pos);
        vel.xy+=0.2*dir*d*1./float(layer_id*5+1);
        
      }
      if(iMouse.z<-0.5){
        float d=1.-smoothstep(particle_size,particle_size*8.,length(iMouse.xy-pos)); //distance to mouse click
        vec2 dir=vec2(1.,0.); 
        if(length(iMouse.xy-pos)>0.001)dir=normalize(iMouse.xy-pos);
        vel.xy+=-0.2*dir*d*1./float(layer_id*5+1);
      }
      if(length(vel)>0.0001)
      vel=min(length(vel),max_vel)*normalize(vel);
      CUSTOM.xy=vel;
      TRANSFORM[3].xyz=vec3(pos.x,pos.y,0.);
      
     }
}

