shader_type canvas_item;
render_mode blend_premul_alpha,unshaded;

uniform sampler2D iChannel0; // self last frame
// self buffer and image size shoult be bigger or equal to vec2(sqrt(num_particles)) with +1 Y if num_particles not pow of 2

uniform sampler2D iChannel1; // this contain intial position and scale and tile ID, from GDScript

// in this example I use buffer to save and read values
// buffer pixel data RGBA [velX, velY, posX, posY]

// you can read other buffers(sampler2D) in this buffer to react to other particle movements

uniform vec3 iMouse; // [x,y,0 or 1 if left click or -1 if it right click]
uniform vec2 iResolution;
uniform bool reset;
uniform int layer_id; // used to control vel multiplier

const ivec2 tile_texture_map_size=ivec2(256,192)/32; // (256,192) is tiles.png size

const float num_particles=20.; // number of particles total
const float particle_size=64.; // in pixels

const float max_vel=4.; // in pixels per frame 1 pixel per frame means 60 pixels per sec
const float vel_slowdown=1.5; // in % example 5 means 5% slowdown every frame

const float pi = 3.1415926;
const float degree_to_rad = pi / 180.0;

int get_index(in ivec2 idx, in ivec2 size){
    return (idx.x%size.x)+idx.y*size.x;
}

ivec2 get_index2(in int idx, in ivec2 size){
    return ivec2(idx%size.x,idx/size.x);
}

void fragment(){
    ivec2 IDX=ivec2(FRAGCOORD.xy);
    int INDEX=get_index(IDX,ivec2(int(sqrt(num_particles))));
    vec4 CUSTOM_local=texelFetch(iChannel0,IDX,0);
      // I use CUSTOM.w as init, CUSTOM==vec4(0.) on first frame
    if(reset)
    {
      vec4 data=texelFetch(iChannel1,IDX,0);
      //init
      CUSTOM_local.xy=vec2(0.); //init vel
      
      vec2 pos=data.xy; //init pos
      
      CUSTOM_local.zw=pos;
      
     }
    else{
      
      vec2 pos=CUSTOM_local.zw;
      vec2 vel=CUSTOM_local.xy;
      pos+=vel;
      
      
      //slowdown vel
      vel*=1.-vel_slowdown/100.;
      
      //mouse clicked
      if(iMouse.z>0.5){
        float d=1.-smoothstep(particle_size,particle_size*8.,length(iMouse.xy-pos)); //distance to mouse click 
        vec2 dir=vec2(1.,0.); 
        if(length(iMouse.xy-pos)>0.001)dir=normalize(iMouse.xy-pos);
        vel.xy+=0.2*dir*d*1./float(layer_id*5+1);
        
      }
      if(iMouse.z<-0.5){
        float d=1.-smoothstep(particle_size,particle_size*8.,length(iMouse.xy-pos)); //distance to mouse click
        vec2 dir=vec2(1.,0.); 
        if(length(iMouse.xy-pos)>0.001)dir=normalize(iMouse.xy-pos);
        vel.xy+=-0.2*dir*d*1./float(layer_id*5+1);
      }
      if(length(vel)>0.0001)
      vel=min(length(vel),max_vel)*normalize(vel);
      CUSTOM_local.xy=vel;
      CUSTOM_local.zw=pos;
     }
    
    COLOR=CUSTOM_local;
}
